/****************************************************************************
** Meta object code from reading C++ file 'dfontpreviewlistview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../interfaces/dfontpreviewlistview.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dfontpreviewlistview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20DFontPreviewListViewE_t {};
} // unnamed namespace

template <> constexpr inline auto DFontPreviewListView::qt_create_metaobjectdata<qt_meta_tag_ZN20DFontPreviewListViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DFontPreviewListView",
        "onLoadFontsStatus",
        "",
        "type",
        "multiItemsAdded",
        "QList<DFontPreviewItemData>&",
        "data",
        "DFontSpinnerWidget::SpinnerStyles",
        "styles",
        "itemRemoved",
        "DFontPreviewItemData&",
        "itemsSelected",
        "files",
        "isFirstInstall",
        "requestInstFontsUiAdded",
        "rowCountChanged",
        "deleteFinished",
        "requestUpdateModel",
        "deleteCount",
        "showSpinner",
        "requestShowSpinner",
        "bShow",
        "force",
        "style",
        "loadUserAddFont",
        "refreshListview",
        "signalHandleDisableTTC",
        "filePath",
        "bool&",
        "isEnable",
        "isConfirm",
        "isAapplyToAll",
        "onEnableBtnClicked",
        "QModelIndexList&",
        "itemIndexes",
        "systemCnt",
        "curCnt",
        "setValue",
        "isFromActiveFont",
        "onCollectBtnClicked",
        "indexList",
        "isFromCollectFont",
        "onListViewShowContextMenu",
        "onFinishedDataLoad",
        "selectFonts",
        "fileList",
        "onMultiItemsAdded",
        "onStartupMultiItemAdded",
        "onUpdateCurrentFont",
        "onFontChanged",
        "QFont",
        "font",
        "onItemRemoved",
        "itemData",
        "updateModel",
        "viewChanged",
        "cancelDel",
        "loadLeftFonts",
        "onRefreshListview"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'onLoadFontsStatus'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'multiItemsAdded'
        QtMocHelpers::SignalData<void(QList<DFontPreviewItemData> &, DFontSpinnerWidget::SpinnerStyles)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'itemRemoved'
        QtMocHelpers::SignalData<void(DFontPreviewItemData &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 6 },
        }}),
        // Signal 'itemsSelected'
        QtMocHelpers::SignalData<void(const QStringList &, bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 12 }, { QMetaType::Bool, 13 },
        }}),
        // Signal 'itemsSelected'
        QtMocHelpers::SignalData<void(const QStringList &)>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QStringList, 12 },
        }}),
        // Signal 'requestInstFontsUiAdded'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rowCountChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'deleteFinished'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestUpdateModel'
        QtMocHelpers::SignalData<void(int, bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::Bool, 19 },
        }}),
        // Signal 'requestShowSpinner'
        QtMocHelpers::SignalData<void(bool, bool, DFontSpinnerWidget::SpinnerStyles)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 21 }, { QMetaType::Bool, 22 }, { 0x80000000 | 7, 23 },
        }}),
        // Signal 'loadUserAddFont'
        QtMocHelpers::SignalData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'refreshListview'
        QtMocHelpers::SignalData<void(QList<DFontPreviewItemData> &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'signalHandleDisableTTC'
        QtMocHelpers::SignalData<void(const QString &, bool &, bool &, bool &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 27 }, { 0x80000000 | 28, 29 }, { 0x80000000 | 28, 30 }, { 0x80000000 | 28, 31 },
        }}),
        // Slot 'onEnableBtnClicked'
        QtMocHelpers::SlotData<void(QModelIndexList &, int, int, bool, bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 34 }, { QMetaType::Int, 35 }, { QMetaType::Int, 36 }, { QMetaType::Bool, 37 },
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'onEnableBtnClicked'
        QtMocHelpers::SlotData<void(QModelIndexList &, int, int, bool)>(32, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 33, 34 }, { QMetaType::Int, 35 }, { QMetaType::Int, 36 }, { QMetaType::Bool, 37 },
        }}),
        // Slot 'onCollectBtnClicked'
        QtMocHelpers::SlotData<void(QModelIndexList &, bool, bool)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 40 }, { QMetaType::Bool, 37 }, { QMetaType::Bool, 41 },
        }}),
        // Slot 'onCollectBtnClicked'
        QtMocHelpers::SlotData<void(QModelIndexList &, bool)>(39, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 33, 40 }, { QMetaType::Bool, 37 },
        }}),
        // Slot 'onListViewShowContextMenu'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onFinishedDataLoad'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectFonts'
        QtMocHelpers::SlotData<void(const QStringList &)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 45 },
        }}),
        // Slot 'onMultiItemsAdded'
        QtMocHelpers::SlotData<void(QList<DFontPreviewItemData> &, DFontSpinnerWidget::SpinnerStyles)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'onStartupMultiItemAdded'
        QtMocHelpers::SlotData<void(QList<DFontPreviewItemData> &)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'onUpdateCurrentFont'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onFontChanged'
        QtMocHelpers::SlotData<void(const QFont &)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 50, 51 },
        }}),
        // Slot 'onItemRemoved'
        QtMocHelpers::SlotData<void(DFontPreviewItemData &)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 53 },
        }}),
        // Slot 'updateModel'
        QtMocHelpers::SlotData<void(int, bool)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::Bool, 19 },
        }}),
        // Slot 'updateModel'
        QtMocHelpers::SlotData<void(int)>(54, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 18 },
        }}),
        // Slot 'viewChanged'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cancelDel'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'loadLeftFonts'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onRefreshListview'
        QtMocHelpers::SlotData<void(QList<DFontPreviewItemData> &)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DFontPreviewListView, qt_meta_tag_ZN20DFontPreviewListViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DFontPreviewListView::staticMetaObject = { {
    QMetaObject::SuperData::link<QListView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20DFontPreviewListViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20DFontPreviewListViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20DFontPreviewListViewE_t>.metaTypes,
    nullptr
} };

void DFontPreviewListView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DFontPreviewListView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onLoadFontsStatus((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->multiItemsAdded((*reinterpret_cast<std::add_pointer_t<QList<DFontPreviewItemData>&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DFontSpinnerWidget::SpinnerStyles>>(_a[2]))); break;
        case 2: _t->itemRemoved((*reinterpret_cast<std::add_pointer_t<DFontPreviewItemData&>>(_a[1]))); break;
        case 3: _t->itemsSelected((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->itemsSelected((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 5: _t->requestInstFontsUiAdded(); break;
        case 6: _t->rowCountChanged(); break;
        case 7: _t->deleteFinished(); break;
        case 8: _t->requestUpdateModel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 9: _t->requestShowSpinner((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<DFontSpinnerWidget::SpinnerStyles>>(_a[3]))); break;
        case 10: _t->loadUserAddFont(); break;
        case 11: _t->refreshListview((*reinterpret_cast<std::add_pointer_t<QList<DFontPreviewItemData>&>>(_a[1]))); break;
        case 12: _t->signalHandleDisableTTC((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[4]))); break;
        case 13: _t->onEnableBtnClicked((*reinterpret_cast<std::add_pointer_t<QModelIndexList&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 14: _t->onEnableBtnClicked((*reinterpret_cast<std::add_pointer_t<QModelIndexList&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 15: _t->onCollectBtnClicked((*reinterpret_cast<std::add_pointer_t<QModelIndexList&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 16: _t->onCollectBtnClicked((*reinterpret_cast<std::add_pointer_t<QModelIndexList&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 17: _t->onListViewShowContextMenu(); break;
        case 18: _t->onFinishedDataLoad(); break;
        case 19: _t->selectFonts((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 20: _t->onMultiItemsAdded((*reinterpret_cast<std::add_pointer_t<QList<DFontPreviewItemData>&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DFontSpinnerWidget::SpinnerStyles>>(_a[2]))); break;
        case 21: _t->onStartupMultiItemAdded((*reinterpret_cast<std::add_pointer_t<QList<DFontPreviewItemData>&>>(_a[1]))); break;
        case 22: _t->onUpdateCurrentFont(); break;
        case 23: _t->onFontChanged((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 24: _t->onItemRemoved((*reinterpret_cast<std::add_pointer_t<DFontPreviewItemData&>>(_a[1]))); break;
        case 25: _t->updateModel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 26: _t->updateModel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 27: _t->viewChanged(); break;
        case 28: _t->cancelDel(); break;
        case 29: _t->loadLeftFonts(); break;
        case 30: _t->onRefreshListview((*reinterpret_cast<std::add_pointer_t<QList<DFontPreviewItemData>&>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListView::*)(int )>(_a, &DFontPreviewListView::onLoadFontsStatus, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListView::*)(QList<DFontPreviewItemData> & , DFontSpinnerWidget::SpinnerStyles )>(_a, &DFontPreviewListView::multiItemsAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListView::*)(DFontPreviewItemData & )>(_a, &DFontPreviewListView::itemRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListView::*)(const QStringList & , bool )>(_a, &DFontPreviewListView::itemsSelected, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListView::*)()>(_a, &DFontPreviewListView::requestInstFontsUiAdded, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListView::*)()>(_a, &DFontPreviewListView::rowCountChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListView::*)()>(_a, &DFontPreviewListView::deleteFinished, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListView::*)(int , bool )>(_a, &DFontPreviewListView::requestUpdateModel, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListView::*)(bool , bool , DFontSpinnerWidget::SpinnerStyles )>(_a, &DFontPreviewListView::requestShowSpinner, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListView::*)()>(_a, &DFontPreviewListView::loadUserAddFont, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListView::*)(QList<DFontPreviewItemData> & )>(_a, &DFontPreviewListView::refreshListview, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListView::*)(const QString & , bool & , bool & , bool & )>(_a, &DFontPreviewListView::signalHandleDisableTTC, 12))
            return;
    }
}

const QMetaObject *DFontPreviewListView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DFontPreviewListView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20DFontPreviewListViewE_t>.strings))
        return static_cast<void*>(this);
    return QListView::qt_metacast(_clname);
}

int DFontPreviewListView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QListView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 31;
    }
    return _id;
}

// SIGNAL 0
void DFontPreviewListView::onLoadFontsStatus(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DFontPreviewListView::multiItemsAdded(QList<DFontPreviewItemData> & _t1, DFontSpinnerWidget::SpinnerStyles _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void DFontPreviewListView::itemRemoved(DFontPreviewItemData & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DFontPreviewListView::itemsSelected(const QStringList & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 5
void DFontPreviewListView::requestInstFontsUiAdded()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void DFontPreviewListView::rowCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void DFontPreviewListView::deleteFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void DFontPreviewListView::requestUpdateModel(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void DFontPreviewListView::requestShowSpinner(bool _t1, bool _t2, DFontSpinnerWidget::SpinnerStyles _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2, _t3);
}

// SIGNAL 10
void DFontPreviewListView::loadUserAddFont()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void DFontPreviewListView::refreshListview(QList<DFontPreviewItemData> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void DFontPreviewListView::signalHandleDisableTTC(const QString & _t1, bool & _t2, bool & _t3, bool & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2, _t3, _t4);
}
QT_WARNING_POP
