/****************************************************************************
** Meta object code from reading C++ file 'dfontpreviewlistdatathread.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../interfaces/dfontpreviewlistdatathread.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dfontpreviewlistdatathread.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26DFontPreviewListDataThreadE_t {};
} // unnamed namespace

template <> constexpr inline auto DFontPreviewListDataThread::qt_create_metaobjectdata<qt_meta_tag_ZN26DFontPreviewListDataThreadE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DFontPreviewListDataThread",
        "requestDeleted",
        "",
        "QStringList&",
        "files",
        "requestAdded",
        "isFirstInstall",
        "requestForceDeleteFiles",
        "requestBatchReInstallContinue",
        "requestRemoveFileWatchers",
        "requestAutoDirWatchers",
        "requestExportFont",
        "exportFontFinished",
        "count",
        "doWork",
        "initFileSystemWatcher",
        "updateChangedDir",
        "addPathWatcher",
        "path",
        "removePathWatcher",
        "onFileDeleted",
        "onFileAdded",
        "forceDeleteFiles",
        "onRemoveFileWatchers",
        "onAutoDirWatchers",
        "onExportFont",
        "fontList"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestDeleted'
        QtMocHelpers::SignalData<void(QStringList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'requestAdded'
        QtMocHelpers::SignalData<void(const QStringList &, bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 4 }, { QMetaType::Bool, 6 },
        }}),
        // Signal 'requestAdded'
        QtMocHelpers::SignalData<void(const QStringList &)>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QStringList, 4 },
        }}),
        // Signal 'requestForceDeleteFiles'
        QtMocHelpers::SignalData<void(QStringList &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'requestBatchReInstallContinue'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestRemoveFileWatchers'
        QtMocHelpers::SignalData<void(const QStringList &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 4 },
        }}),
        // Signal 'requestAutoDirWatchers'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestExportFont'
        QtMocHelpers::SignalData<void(QStringList &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'exportFontFinished'
        QtMocHelpers::SignalData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Slot 'doWork'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'initFileSystemWatcher'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateChangedDir'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addPathWatcher'
        QtMocHelpers::SlotData<void(const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'removePathWatcher'
        QtMocHelpers::SlotData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'onFileDeleted'
        QtMocHelpers::SlotData<void(QStringList &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'onFileAdded'
        QtMocHelpers::SlotData<void(const QStringList &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 4 },
        }}),
        // Slot 'forceDeleteFiles'
        QtMocHelpers::SlotData<void(QStringList &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'onRemoveFileWatchers'
        QtMocHelpers::SlotData<void(const QStringList &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 4 },
        }}),
        // Slot 'onAutoDirWatchers'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onExportFont'
        QtMocHelpers::SlotData<void(QStringList &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DFontPreviewListDataThread, qt_meta_tag_ZN26DFontPreviewListDataThreadE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DFontPreviewListDataThread::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26DFontPreviewListDataThreadE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26DFontPreviewListDataThreadE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26DFontPreviewListDataThreadE_t>.metaTypes,
    nullptr
} };

void DFontPreviewListDataThread::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DFontPreviewListDataThread *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestDeleted((*reinterpret_cast<std::add_pointer_t<QStringList&>>(_a[1]))); break;
        case 1: _t->requestAdded((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 2: _t->requestAdded((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 3: _t->requestForceDeleteFiles((*reinterpret_cast<std::add_pointer_t<QStringList&>>(_a[1]))); break;
        case 4: _t->requestBatchReInstallContinue(); break;
        case 5: _t->requestRemoveFileWatchers((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 6: _t->requestAutoDirWatchers(); break;
        case 7: _t->requestExportFont((*reinterpret_cast<std::add_pointer_t<QStringList&>>(_a[1]))); break;
        case 8: _t->exportFontFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->doWork(); break;
        case 10: _t->initFileSystemWatcher(); break;
        case 11: _t->updateChangedDir(); break;
        case 12: _t->addPathWatcher((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->removePathWatcher((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->onFileDeleted((*reinterpret_cast<std::add_pointer_t<QStringList&>>(_a[1]))); break;
        case 15: _t->onFileAdded((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 16: _t->forceDeleteFiles((*reinterpret_cast<std::add_pointer_t<QStringList&>>(_a[1]))); break;
        case 17: _t->onRemoveFileWatchers((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 18: _t->onAutoDirWatchers(); break;
        case 19: _t->onExportFont((*reinterpret_cast<std::add_pointer_t<QStringList&>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListDataThread::*)(QStringList & )>(_a, &DFontPreviewListDataThread::requestDeleted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListDataThread::*)(const QStringList & , bool )>(_a, &DFontPreviewListDataThread::requestAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListDataThread::*)(QStringList & )>(_a, &DFontPreviewListDataThread::requestForceDeleteFiles, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListDataThread::*)()>(_a, &DFontPreviewListDataThread::requestBatchReInstallContinue, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListDataThread::*)(const QStringList & )>(_a, &DFontPreviewListDataThread::requestRemoveFileWatchers, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListDataThread::*)()>(_a, &DFontPreviewListDataThread::requestAutoDirWatchers, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListDataThread::*)(QStringList & )>(_a, &DFontPreviewListDataThread::requestExportFont, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontPreviewListDataThread::*)(int )>(_a, &DFontPreviewListDataThread::exportFontFinished, 8))
            return;
    }
}

const QMetaObject *DFontPreviewListDataThread::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DFontPreviewListDataThread::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26DFontPreviewListDataThreadE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DFontPreviewListDataThread::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void DFontPreviewListDataThread::requestDeleted(QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DFontPreviewListDataThread::requestAdded(const QStringList & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 3
void DFontPreviewListDataThread::requestForceDeleteFiles(QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void DFontPreviewListDataThread::requestBatchReInstallContinue()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void DFontPreviewListDataThread::requestRemoveFileWatchers(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void DFontPreviewListDataThread::requestAutoDirWatchers()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void DFontPreviewListDataThread::requestExportFont(QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void DFontPreviewListDataThread::exportFontFinished(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
