/****************************************************************************
** Meta object code from reading C++ file 'dfinstallerrordialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../views/dfinstallerrordialog.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dfinstallerrordialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20DFInstallErrorDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto DFInstallErrorDialog::qt_create_metaobjectdata<qt_meta_tag_ZN20DFInstallErrorDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DFInstallErrorDialog",
        "onCancelInstall",
        "",
        "onContinueInstall",
        "continueInstallFontFilelList",
        "onListItemClicked",
        "QModelIndex",
        "index",
        "onListItemsClicked",
        "QModelIndexList",
        "indexList",
        "addData",
        "QStringList&",
        "errorFileList",
        "halfInstalledFilelist",
        "addHalfInstalledFiles",
        "oldHalfInstalledFiles",
        "onControlButtonClicked",
        "btnIndex",
        "slotSizeModeChanged",
        "DGuiApplicationHelper::SizeMode",
        "sizeMode"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'onCancelInstall'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'onContinueInstall'
        QtMocHelpers::SignalData<void(const QStringList &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 4 },
        }}),
        // Slot 'onListItemClicked'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'onListItemsClicked'
        QtMocHelpers::SlotData<void(const QModelIndexList &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'addData'
        QtMocHelpers::SlotData<void(QStringList &, QStringList &, QStringList &, QStringList &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { 0x80000000 | 12, 14 }, { 0x80000000 | 12, 15 }, { 0x80000000 | 12, 16 },
        }}),
        // Slot 'onControlButtonClicked'
        QtMocHelpers::SlotData<void(int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 },
        }}),
        // Slot 'slotSizeModeChanged'
        QtMocHelpers::SlotData<void(DGuiApplicationHelper::SizeMode)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DFInstallErrorDialog, qt_meta_tag_ZN20DFInstallErrorDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DFInstallErrorDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20DFInstallErrorDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20DFInstallErrorDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20DFInstallErrorDialogE_t>.metaTypes,
    nullptr
} };

void DFInstallErrorDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DFInstallErrorDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onCancelInstall(); break;
        case 1: _t->onContinueInstall((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 2: _t->onListItemClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 3: _t->onListItemsClicked((*reinterpret_cast<std::add_pointer_t<QModelIndexList>>(_a[1]))); break;
        case 4: _t->addData((*reinterpret_cast<std::add_pointer_t<QStringList&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStringList&>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QStringList&>>(_a[4]))); break;
        case 5: _t->onControlButtonClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->slotSizeModeChanged((*reinterpret_cast<std::add_pointer_t<DGuiApplicationHelper::SizeMode>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QModelIndexList >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DFInstallErrorDialog::*)()>(_a, &DFInstallErrorDialog::onCancelInstall, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFInstallErrorDialog::*)(const QStringList & )>(_a, &DFInstallErrorDialog::onContinueInstall, 1))
            return;
    }
}

const QMetaObject *DFInstallErrorDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DFInstallErrorDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20DFInstallErrorDialogE_t>.strings))
        return static_cast<void*>(this);
    return DDialog::qt_metacast(_clname);
}

int DFInstallErrorDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void DFInstallErrorDialog::onCancelInstall()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DFInstallErrorDialog::onContinueInstall(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
