/****************************************************************************
** Meta object code from reading C++ file 'dfontmgrmainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../views/dfontmgrmainwindow.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dfontmgrmainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18DFontMgrMainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto DFontMgrMainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN18DFontMgrMainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DFontMgrMainWindow",
        "fileSelected",
        "",
        "files",
        "isAddBtnHasTabs",
        "fileSelectedInSys",
        "quickModeInstall",
        "singalLoadLeftFontsFinsih",
        "handleMenuEvent",
        "QAction*",
        "onSearchTextChanged",
        "currStr",
        "onPreviewTextChanged",
        "text",
        "onFontSizeChanged",
        "fontSize",
        "onLeftSiderBarItemClicked",
        "index",
        "needClearSelect",
        "onFontInstallFinished",
        "fileList",
        "onRefreshUserFont",
        "onUninstallFcCacheFinish",
        "onFontListViewRowCountChanged",
        "onLoadStatus",
        "type",
        "onShowMessage",
        "totalCount",
        "success",
        "onShowSpinner",
        "bShow",
        "force",
        "DFontSpinnerWidget::SpinnerStyles",
        "style",
        "onconfirmDelDlgAccept",
        "onInstallWindowDestroyed",
        "onRightMenuAboutToShow",
        "onRightMenuAboutToHide",
        "onUninstallFontFinished",
        "QStringList&",
        "onCacheFinish",
        "onHandleDeleteTTC",
        "filePath",
        "bool&",
        "isDelete",
        "isAapplyToAll",
        "onRequestInstFontsUiAdded",
        "onFontChanged",
        "onFontSizeRequestToSlider",
        "onExportFontFinished",
        "count",
        "onMenuHidden",
        "onFontManagerFinished",
        "onHandleDisableTTC",
        "isEnable",
        "isConfirm",
        "slotSizeModeChanged",
        "DGuiApplicationHelper::SizeMode",
        "sizeMode"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fileSelected'
        QtMocHelpers::SignalData<void(const QStringList &, bool) const>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 3 }, { QMetaType::Bool, 4 },
        }}),
        // Signal 'fileSelectedInSys'
        QtMocHelpers::SignalData<void(const QStringList &) const>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 3 },
        }}),
        // Signal 'quickModeInstall'
        QtMocHelpers::SignalData<void(const QStringList &) const>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 3 },
        }}),
        // Signal 'singalLoadLeftFontsFinsih'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleMenuEvent'
        QtMocHelpers::SlotData<void(QAction *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 },
        }}),
        // Slot 'onSearchTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'onPreviewTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'onFontSizeChanged'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'onLeftSiderBarItemClicked'
        QtMocHelpers::SlotData<void(int, bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 17 }, { QMetaType::Bool, 18 },
        }}),
        // Slot 'onLeftSiderBarItemClicked'
        QtMocHelpers::SlotData<void(int)>(16, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 17 },
        }}),
        // Slot 'onLeftSiderBarItemClicked'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'onFontInstallFinished'
        QtMocHelpers::SlotData<void(const QStringList &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 20 },
        }}),
        // Slot 'onRefreshUserFont'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onUninstallFcCacheFinish'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onFontListViewRowCountChanged'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onLoadStatus'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'onShowMessage'
        QtMocHelpers::SlotData<void(int, bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 27 }, { QMetaType::Bool, 28 },
        }}),
        // Slot 'onShowMessage'
        QtMocHelpers::SlotData<void(int)>(26, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'onShowSpinner'
        QtMocHelpers::SlotData<void(bool, bool, DFontSpinnerWidget::SpinnerStyles)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 30 }, { QMetaType::Bool, 31 }, { 0x80000000 | 32, 33 },
        }}),
        // Slot 'onconfirmDelDlgAccept'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onInstallWindowDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 2 },
        }}),
        // Slot 'onInstallWindowDestroyed'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'onRightMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onRightMenuAboutToHide'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onUninstallFontFinished'
        QtMocHelpers::SlotData<void(QStringList &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 39, 3 },
        }}),
        // Slot 'onCacheFinish'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onHandleDeleteTTC'
        QtMocHelpers::SlotData<void(QString, bool &, bool &)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 42 }, { 0x80000000 | 43, 44 }, { 0x80000000 | 43, 45 },
        }}),
        // Slot 'onRequestInstFontsUiAdded'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onFontChanged'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onFontSizeRequestToSlider'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onExportFontFinished'
        QtMocHelpers::SlotData<void(int)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 50 },
        }}),
        // Slot 'onMenuHidden'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onFontManagerFinished'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onHandleDisableTTC'
        QtMocHelpers::SlotData<void(const QString &, bool &, bool &, bool &)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 42 }, { 0x80000000 | 43, 54 }, { 0x80000000 | 43, 55 }, { 0x80000000 | 43, 45 },
        }}),
        // Slot 'slotSizeModeChanged'
        QtMocHelpers::SlotData<void(DGuiApplicationHelper::SizeMode)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 57, 58 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DFontMgrMainWindow, qt_meta_tag_ZN18DFontMgrMainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DFontMgrMainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<DMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18DFontMgrMainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18DFontMgrMainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18DFontMgrMainWindowE_t>.metaTypes,
    nullptr
} };

void DFontMgrMainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DFontMgrMainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fileSelected((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->fileSelectedInSys((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 2: _t->quickModeInstall((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 3: _t->singalLoadLeftFontsFinsih(); break;
        case 4: _t->handleMenuEvent((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 5: _t->onSearchTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->onPreviewTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->onFontSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->onLeftSiderBarItemClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 9: _t->onLeftSiderBarItemClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->onLeftSiderBarItemClicked(); break;
        case 11: _t->onFontInstallFinished((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 12: _t->onRefreshUserFont(); break;
        case 13: _t->onUninstallFcCacheFinish(); break;
        case 14: _t->onFontListViewRowCountChanged(); break;
        case 15: _t->onLoadStatus((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->onShowMessage((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 17: _t->onShowMessage((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->onShowSpinner((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<DFontSpinnerWidget::SpinnerStyles>>(_a[3]))); break;
        case 19: _t->onconfirmDelDlgAccept(); break;
        case 20: _t->onInstallWindowDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 21: _t->onInstallWindowDestroyed(); break;
        case 22: _t->onRightMenuAboutToShow(); break;
        case 23: _t->onRightMenuAboutToHide(); break;
        case 24: _t->onUninstallFontFinished((*reinterpret_cast<std::add_pointer_t<QStringList&>>(_a[1]))); break;
        case 25: _t->onCacheFinish(); break;
        case 26: _t->onHandleDeleteTTC((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[3]))); break;
        case 27: _t->onRequestInstFontsUiAdded(); break;
        case 28: _t->onFontChanged(); break;
        case 29: _t->onFontSizeRequestToSlider(); break;
        case 30: _t->onExportFontFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 31: _t->onMenuHidden(); break;
        case 32: _t->onFontManagerFinished(); break;
        case 33: _t->onHandleDisableTTC((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[4]))); break;
        case 34: _t->slotSizeModeChanged((*reinterpret_cast<std::add_pointer_t<DGuiApplicationHelper::SizeMode>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DFontMgrMainWindow::*)(const QStringList & , bool ) const>(_a, &DFontMgrMainWindow::fileSelected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontMgrMainWindow::*)(const QStringList & ) const>(_a, &DFontMgrMainWindow::fileSelectedInSys, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontMgrMainWindow::*)(const QStringList & ) const>(_a, &DFontMgrMainWindow::quickModeInstall, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFontMgrMainWindow::*)()>(_a, &DFontMgrMainWindow::singalLoadLeftFontsFinsih, 3))
            return;
    }
}

const QMetaObject *DFontMgrMainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DFontMgrMainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18DFontMgrMainWindowE_t>.strings))
        return static_cast<void*>(this);
    return DMainWindow::qt_metacast(_clname);
}

int DFontMgrMainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    return _id;
}

// SIGNAL 0
void DFontMgrMainWindow::fileSelected(const QStringList & _t1, bool _t2)const
{
    QMetaObject::activate<void>(const_cast< DFontMgrMainWindow *>(this), &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void DFontMgrMainWindow::fileSelectedInSys(const QStringList & _t1)const
{
    QMetaObject::activate<void>(const_cast< DFontMgrMainWindow *>(this), &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DFontMgrMainWindow::quickModeInstall(const QStringList & _t1)const
{
    QMetaObject::activate<void>(const_cast< DFontMgrMainWindow *>(this), &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DFontMgrMainWindow::singalLoadLeftFontsFinsih()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
