/****************************************************************************
** Meta object code from reading C++ file 'signalmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../libdeepin-font-manager/signalmanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'signalmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13SignalManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto SignalManager::qt_create_metaobjectdata<qt_meta_tag_ZN13SignalManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SignalManager",
        "finishFontInstall",
        "",
        "fileList",
        "refreshUserFont",
        "sizeChange",
        "height",
        "popInstallErrorDialog",
        "hideInstallErrorDialog",
        "updateInstallErrorListview",
        "QStringList&",
        "errorFileList",
        "halfInstalledFiles",
        "addHalfInstalledFiles",
        "oldHalfInstalledFiles",
        "installDuringPopErrorDialog",
        "setSpliteWidgetScrollEnable",
        "isInstalling",
        "changeView",
        "cancelDel",
        "fontSizeRequestToSlider",
        "requestSetLeftSiderBarFocus",
        "clearRecoverList",
        "setLostFocusState",
        "isTrue",
        "requestSetTabFocusToAddBtn",
        "menuHidden"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'finishFontInstall'
        QtMocHelpers::SignalData<void(const QStringList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 3 },
        }}),
        // Signal 'refreshUserFont'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sizeChange'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'popInstallErrorDialog'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hideInstallErrorDialog'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateInstallErrorListview'
        QtMocHelpers::SignalData<void(QStringList &, QStringList &, QStringList &, QStringList &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 10, 12 }, { 0x80000000 | 10, 13 }, { 0x80000000 | 10, 14 },
        }}),
        // Signal 'installDuringPopErrorDialog'
        QtMocHelpers::SignalData<void(const QStringList &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 3 },
        }}),
        // Signal 'setSpliteWidgetScrollEnable'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Signal 'changeView'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cancelDel'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fontSizeRequestToSlider'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestSetLeftSiderBarFocus'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clearRecoverList'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'setLostFocusState'
        QtMocHelpers::SignalData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 24 },
        }}),
        // Signal 'requestSetTabFocusToAddBtn'
        QtMocHelpers::SignalData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'menuHidden'
        QtMocHelpers::SignalData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SignalManager, qt_meta_tag_ZN13SignalManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SignalManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13SignalManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13SignalManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13SignalManagerE_t>.metaTypes,
    nullptr
} };

void SignalManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SignalManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->finishFontInstall((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 1: _t->refreshUserFont(); break;
        case 2: _t->sizeChange((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->popInstallErrorDialog(); break;
        case 4: _t->hideInstallErrorDialog(); break;
        case 5: _t->updateInstallErrorListview((*reinterpret_cast<std::add_pointer_t<QStringList&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStringList&>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QStringList&>>(_a[4]))); break;
        case 6: _t->installDuringPopErrorDialog((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 7: _t->setSpliteWidgetScrollEnable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->changeView(); break;
        case 9: _t->cancelDel(); break;
        case 10: _t->fontSizeRequestToSlider(); break;
        case 11: _t->requestSetLeftSiderBarFocus(); break;
        case 12: _t->clearRecoverList(); break;
        case 13: _t->setLostFocusState((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->requestSetTabFocusToAddBtn(); break;
        case 15: _t->menuHidden(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SignalManager::*)(const QStringList & )>(_a, &SignalManager::finishFontInstall, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SignalManager::*)()>(_a, &SignalManager::refreshUserFont, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SignalManager::*)(int )>(_a, &SignalManager::sizeChange, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SignalManager::*)()>(_a, &SignalManager::popInstallErrorDialog, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SignalManager::*)()>(_a, &SignalManager::hideInstallErrorDialog, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SignalManager::*)(QStringList & , QStringList & , QStringList & , QStringList & )>(_a, &SignalManager::updateInstallErrorListview, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (SignalManager::*)(const QStringList & )>(_a, &SignalManager::installDuringPopErrorDialog, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (SignalManager::*)(bool )>(_a, &SignalManager::setSpliteWidgetScrollEnable, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (SignalManager::*)()>(_a, &SignalManager::changeView, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (SignalManager::*)()>(_a, &SignalManager::cancelDel, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (SignalManager::*)()>(_a, &SignalManager::fontSizeRequestToSlider, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (SignalManager::*)()>(_a, &SignalManager::requestSetLeftSiderBarFocus, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (SignalManager::*)()>(_a, &SignalManager::clearRecoverList, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (SignalManager::*)(bool )>(_a, &SignalManager::setLostFocusState, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (SignalManager::*)()>(_a, &SignalManager::requestSetTabFocusToAddBtn, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (SignalManager::*)()>(_a, &SignalManager::menuHidden, 15))
            return;
    }
}

const QMetaObject *SignalManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SignalManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13SignalManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int SignalManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void SignalManager::finishFontInstall(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void SignalManager::refreshUserFont()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void SignalManager::sizeChange(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void SignalManager::popInstallErrorDialog()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void SignalManager::hideInstallErrorDialog()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void SignalManager::updateInstallErrorListview(QStringList & _t1, QStringList & _t2, QStringList & _t3, QStringList & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 6
void SignalManager::installDuringPopErrorDialog(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void SignalManager::setSpliteWidgetScrollEnable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void SignalManager::changeView()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void SignalManager::cancelDel()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void SignalManager::fontSizeRequestToSlider()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void SignalManager::requestSetLeftSiderBarFocus()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void SignalManager::clearRecoverList()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void SignalManager::setLostFocusState(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void SignalManager::requestSetTabFocusToAddBtn()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void SignalManager::menuHidden()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}
QT_WARNING_POP
