/****************************************************************************
** Meta object code from reading C++ file 'dfinstallnormalwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../views/dfinstallnormalwindow.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dfinstallnormalwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21DFInstallNormalWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto DFInstallNormalWindow::qt_create_metaobjectdata<qt_meta_tag_ZN21DFInstallNormalWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DFInstallNormalWindow",
        "finishFontInstall",
        "",
        "fileList",
        "sigdowork",
        "DFInstallNormalWindow*",
        "w",
        "sigShowInstallErrDlg",
        "batchInstall",
        "batchHalfwayInstall",
        "filelist",
        "batchReInstallContinue",
        "onProgressChanged",
        "familyName",
        "percent",
        "onInstallFinished",
        "state",
        "onReInstallFinished",
        "onCancelInstall",
        "onContinueInstall",
        "continueInstallFontFileList",
        "showInstallErrDlg"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'finishFontInstall'
        QtMocHelpers::SignalData<void(const QStringList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 3 },
        }}),
        // Signal 'sigdowork'
        QtMocHelpers::SignalData<void(DFInstallNormalWindow *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'sigShowInstallErrDlg'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'batchInstall'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'batchHalfwayInstall'
        QtMocHelpers::SlotData<void(const QStringList &)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QStringList, 10 },
        }}),
        // Slot 'batchReInstallContinue'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onProgressChanged'
        QtMocHelpers::SlotData<void(const QString &, const double &)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { QMetaType::Double, 14 },
        }}),
        // Slot 'onInstallFinished'
        QtMocHelpers::SlotData<void(int, const QStringList &)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 16 }, { QMetaType::QStringList, 3 },
        }}),
        // Slot 'onReInstallFinished'
        QtMocHelpers::SlotData<void(int, const QStringList &)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 16 }, { QMetaType::QStringList, 3 },
        }}),
        // Slot 'onCancelInstall'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onContinueInstall'
        QtMocHelpers::SlotData<void(const QStringList &)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QStringList, 20 },
        }}),
        // Slot 'showInstallErrDlg'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DFInstallNormalWindow, qt_meta_tag_ZN21DFInstallNormalWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DFInstallNormalWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<DFontBaseDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DFInstallNormalWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DFInstallNormalWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21DFInstallNormalWindowE_t>.metaTypes,
    nullptr
} };

void DFInstallNormalWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DFInstallNormalWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->finishFontInstall((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 1: _t->sigdowork((*reinterpret_cast<std::add_pointer_t<DFInstallNormalWindow*>>(_a[1]))); break;
        case 2: _t->sigShowInstallErrDlg(); break;
        case 3: _t->batchInstall(); break;
        case 4: _t->batchHalfwayInstall((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 5: _t->batchReInstallContinue(); break;
        case 6: _t->onProgressChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 7: _t->onInstallFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 8: _t->onReInstallFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 9: _t->onCancelInstall(); break;
        case 10: _t->onContinueInstall((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 11: _t->showInstallErrDlg(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DFInstallNormalWindow* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DFInstallNormalWindow::*)(const QStringList & )>(_a, &DFInstallNormalWindow::finishFontInstall, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFInstallNormalWindow::*)(DFInstallNormalWindow * )>(_a, &DFInstallNormalWindow::sigdowork, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DFInstallNormalWindow::*)()>(_a, &DFInstallNormalWindow::sigShowInstallErrDlg, 2))
            return;
    }
}

const QMetaObject *DFInstallNormalWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DFInstallNormalWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DFInstallNormalWindowE_t>.strings))
        return static_cast<void*>(this);
    return DFontBaseDialog::qt_metacast(_clname);
}

int DFInstallNormalWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DFontBaseDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void DFInstallNormalWindow::finishFontInstall(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DFInstallNormalWindow::sigdowork(DFInstallNormalWindow * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DFInstallNormalWindow::sigShowInstallErrDlg()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
