/****************************************************************************
** Meta object code from reading C++ file 'fontmanagercore.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../libdeepin-font-manager/fontmanagercore.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fontmanagercore.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15FontManagerCoreE_t {};
} // unnamed namespace

template <> constexpr inline auto FontManagerCore::qt_create_metaobjectdata<qt_meta_tag_ZN15FontManagerCoreE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FontManagerCore",
        "batchInstall",
        "",
        "filePath",
        "percent",
        "installFinished",
        "state",
        "fileList",
        "reInstallFinished",
        "uninstallFontFinished",
        "QStringList&",
        "uninstallIndex",
        "uninstallFcCacheFinish",
        "cacheFinish",
        "requestCancelInstall",
        "handleDeleteTTC",
        "file",
        "bool&",
        "isDelete",
        "isAapplyToAll",
        "onInstallResult",
        "familyName",
        "target"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'batchInstall'
        QtMocHelpers::SignalData<void(const QString &, const double &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Double, 4 },
        }}),
        // Signal 'installFinished'
        QtMocHelpers::SignalData<void(int, const QStringList &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { QMetaType::QStringList, 7 },
        }}),
        // Signal 'reInstallFinished'
        QtMocHelpers::SignalData<void(int, const QStringList &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { QMetaType::QStringList, 7 },
        }}),
        // Signal 'uninstallFontFinished'
        QtMocHelpers::SignalData<void(QStringList &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'uninstallFcCacheFinish'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cacheFinish'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestCancelInstall'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'handleDeleteTTC'
        QtMocHelpers::SignalData<void(QString, bool &, bool &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 }, { 0x80000000 | 17, 18 }, { 0x80000000 | 17, 19 },
        }}),
        // Slot 'onInstallResult'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 }, { QMetaType::QString, 22 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FontManagerCore, qt_meta_tag_ZN15FontManagerCoreE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FontManagerCore::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15FontManagerCoreE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15FontManagerCoreE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15FontManagerCoreE_t>.metaTypes,
    nullptr
} };

void FontManagerCore::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FontManagerCore *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->batchInstall((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 1: _t->installFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 2: _t->reInstallFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 3: _t->uninstallFontFinished((*reinterpret_cast<std::add_pointer_t<QStringList&>>(_a[1]))); break;
        case 4: _t->uninstallFcCacheFinish(); break;
        case 5: _t->cacheFinish(); break;
        case 6: _t->requestCancelInstall(); break;
        case 7: _t->handleDeleteTTC((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[3]))); break;
        case 8: _t->onInstallResult((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FontManagerCore::*)(const QString & , const double & )>(_a, &FontManagerCore::batchInstall, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FontManagerCore::*)(int , const QStringList & )>(_a, &FontManagerCore::installFinished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FontManagerCore::*)(int , const QStringList & )>(_a, &FontManagerCore::reInstallFinished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FontManagerCore::*)(QStringList & )>(_a, &FontManagerCore::uninstallFontFinished, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (FontManagerCore::*)()>(_a, &FontManagerCore::uninstallFcCacheFinish, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (FontManagerCore::*)()>(_a, &FontManagerCore::cacheFinish, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (FontManagerCore::*)()>(_a, &FontManagerCore::requestCancelInstall, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (FontManagerCore::*)(QString , bool & , bool & )>(_a, &FontManagerCore::handleDeleteTTC, 7))
            return;
    }
}

const QMetaObject *FontManagerCore::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FontManagerCore::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15FontManagerCoreE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int FontManagerCore::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void FontManagerCore::batchInstall(const QString & _t1, const double & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void FontManagerCore::installFinished(int _t1, const QStringList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void FontManagerCore::reInstallFinished(int _t1, const QStringList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void FontManagerCore::uninstallFontFinished(QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void FontManagerCore::uninstallFcCacheFinish()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void FontManagerCore::cacheFinish()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void FontManagerCore::requestCancelInstall()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void FontManagerCore::handleDeleteTTC(QString _t1, bool & _t2, bool & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
